<?php

namespace InterludeSante\Legal;

use ACFComposer\ACFComposer;
use InterludeSante\Legal\stdClass;

class Legal
{

    /**
     * The single instance of the class.
     *
     * @since  1.0
     * @access protected
     */
    protected static $instance = null;

    /**
     * Main instance.
     *
     * Ensures only one instance is loaded or can be loaded.
     *
     * @since  1.0
     * @access public
     * @param  string $path
     *
     * @return Main instance.
     */
    public static function instance()
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function __construct()
    {
        $this->registerHooks();
    }

    /**
     * Retrieve the name of the plugin.
     *
     * @since  1.0
     * @access public
     *
     * @return string
     */
    public function getName()
    {
        $plugin_data = get_file_data($this->path, ['Plugin Name' => 'Plugin Name'], 'plugin');
        return $plugin_data['Plugin Name'];
    }

    /**
     * Registering WordPress Hooks.
     *
     * @since  1.0
     * @access public
     */
    public function registerHooks()
    {
        add_action('admin_menu', [$this, 'addOptionPage']);

        add_action('init', [$this, 'registerCGUFields']);
        add_action('init', [$this, 'registerRGPDFields']);
        add_action('init', [$this, 'registerLegalFields']);

        add_action('rest_api_init', [$this, 'registerRouteApi']);
    }

    /**
     * Create sub menu page of the plugin.
     *
     * @since  1.0
     * @access public
     */
    public function addOptionPage()
    {
        $parent = acf_add_options_page(array(
            'page_title'  => __('Documents administratifs'),
            'menu_title'  => __('Documents administratifs'),
            'redirect'    => true,
        ));

        acf_add_options_page(array(
            'page_title'  => __('CGU'),
            'menu_title'  => __('CGU'),
            'menu_slug'     => 'menu-cgu',
            'parent_slug' => $parent['menu_slug'],
        ));

        acf_add_options_page(array(
            'page_title'  => __('RGPD'),
            'menu_title'  => __('RGPD'),
            'menu_slug'     => 'menu-rgpd',
            'parent_slug' => $parent['menu_slug'],
        ));

        acf_add_options_page(array(
            'page_title'  => __('Mentions Légales'),
            'menu_title'  => __('Mentions Légales'),
            'menu_slug'     => 'menu-legal',
            'parent_slug' => $parent['menu_slug'],
        ));
    }



    /**
     * Registering CGU ACF field.
     *
     * @since  1.0
     * @access public
     */
    public function registerCGUFields()
    {
        ACFComposer::registerFieldGroup([
            'name' => 'cgu',
            'title' => 'cgu',
            'style' => 'seamless',
            'fields' =>
            [
               [
                   'label' => 'Zone de texte des CGU',
                   'name' => 'cgu-content',
                   'type' => 'wysiwyg',
                   'tabs' => 'visual,text',
                   'toolbar' => 'default',
                   'media_upload' => 0,
                   'delay' => 1
               ]
            ],
            'location' => [
                [
                    [
                        'param' => 'options_page',
                        'operator' => '==',
                        'value' => 'menu-cgu',
                    ],
                ],
            ],
        ]);
    }

    /**
     * Registering RGPD ACF field.
     *
     * @since  1.0
     * @access public
     */
    public function registerRGPDFields()
    {
        ACFComposer::registerFieldGroup([
            'name' => 'rgpd',
            'title' => 'rgpd',
            'style' => 'seamless',
            'fields' =>
            [
               [
                   'label' => 'Zone de texte des RGPD',
                   'name' => 'rgpd-content',
                   'type' => 'wysiwyg',
                   'tabs' => 'visual,text',
                   'toolbar' => 'default',
                   'media_upload' => 0,
                   'delay' => 1
               ]
            ],
            'location' => [
                [
                    [
                        'param' => 'options_page',
                        'operator' => '==',
                        'value' => 'menu-rgpd',
                    ],
                ],
            ],
        ]);
    }

    /**
         * Registering Legal Content ACF field.
         *
         * @since  1.0
         * @access public
         */
    public function registerLegalFields()
    {
        ACFComposer::registerFieldGroup([
            'name' => 'legal',
            'title' => 'legal',
            'style' => 'seamless',
            'fields' =>
            [
               [
                   'label' => 'Zone de texte des mentions légales',
                   'name' => 'legal-content',
                   'type' => 'wysiwyg',
                   'tabs' => 'visual,text',
                   'toolbar' => 'default',
                   'media_upload' => 0,
                   'delay' => 1
               ]
            ],
            'location' => [
                [
                    [
                        'param' => 'options_page',
                        'operator' => '==',
                        'value' => 'menu-legal',
                    ],
                ],
            ],
        ]);
    }

    public function registerRouteApi()
    {
        register_rest_route('mobile', '/cgu', array(
            'methods' => 'GET',
            'callback' => [$this, 'get_cgu'],
        ));

        register_rest_route('mobile', '/rgpd', array(
            'methods' => 'GET',
            'callback' => [$this, 'get_rgpd'],
        ));

        register_rest_route('mobile', '/legal', array(
            'methods' => 'GET',
            'callback' => [$this, 'get_legal'],
        ));
    }

    public function get_cgu($request_data)
    {
        $$object;
        $object->code = "success";
        $object->data->title = "Conditions générales d'utilisation";
        $object->data->content = get_field("cgu-content", "option");
        return $object;
    }

    public function get_rgpd($request_data)
    {
        $$object;
        $object->code = "success";
        $object->data->title = "RGPD";
        $object->data->content = get_field("rgpd-content", "option");
        return $object;
    }

    public function get_legal($request_data)
    {
        $$object;
        $object->code = "success";
        $object->data->title = "Mentions légales";
        $object->data->content = get_field("legal-content", "option");

        return $object;
    }
}
